/*
 Name:		QSLA_PONG_FP2.ino
 Created:	4/6/2018 3:42:57 PM
 Author:	Kirschner Christoph
*/




#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_IS31FL3731.h>



// If you're using the full breakout...
//Adafruit_IS31FL3731 ledmatrix = Adafruit_IS31FL3731();
// If you're using the FeatherWing version
Adafruit_IS31FL3731_Wing ledmatrix = Adafruit_IS31FL3731_Wing();


#define ButtonLeft 4
#define ButtonRight 5
#define PADSIZE 3
#define BALL_DELAY 200
#define GAME_DELAY 10
#define BOUNCE_VERTICAL 1
#define BOUNCE_HORIZONTAL -1
#define NEW_GAME_ANIMATION_SPEED 50
#define HIT_NONE 0
#define HIT_CENTER 1
#define HIT_LEFT 2
#define HIT_RIGHT 3

int lastPressButtonLeft  = 0;
int firstPressed;
int pressingTime;
bool lastState;
int lastPressButtonRight = 0;
int lastBallMove = 0;
byte padmap;

//#define DEBUG 1

byte sad[] = {
	B00000000,
	B01000100,
	B00010000,
	B00010000,
	B00000000,
	B00111000,
	B01000100,
	B00000000
};

byte smile[] = {
	B00000000,
	B01000100,
	B00000000,
	B00010000,
	B00010000,
	B01000100,
	B00111000,
	B00000000
};




byte direction; // Wind rose, 0 is north
int xball;
int yball;

int xballOld1;
int yballOld1;
int xballOld2;
int yballOld2;

int xpad = 1;
int ball_timer;

void setup() {

	Serial.begin(9600);
	//while (!Serial);
	Serial.println("ISSI manual animation test");
	if (!ledmatrix.begin()) {
		Serial.println("IS31 not found");
		while (1);
}
	Serial.println("IS31 Found!");

	ledmatrix.setTextWrap(false);
	ledmatrix.setTextColor(255); // quarter brightness


	randomSeed(analogRead(0));

	pinMode(ButtonLeft, INPUT_PULLUP);
	pinMode(ButtonRight, INPUT_PULLUP);

	Serial.begin(9600);
	Serial.println("Pong");
	ledmatrix.clear();
	ledmatrix.setCursor(0, 0);
	ledmatrix.setTextSize(1);
	ledmatrix.setTextWrap(false);  // we dont want text to wrap so it scrolls nicely
	ledmatrix.setTextColor(255);
	for (int8_t x = 0; x >= -60; x--) {
		ledmatrix.clear();
		ledmatrix.setCursor(x, 0);
		ledmatrix.print("PONG FOR FP2");
		if (digitalRead(ButtonLeft) == LOW || digitalRead(ButtonRight) == LOW) {
			goto skipStartAnimation;
		}
		delay(100);
	}

	skipStartAnimation:
	ledmatrix.clear();

	newGame();
}

void loop() {

	// Move pad
	setPad();

	Serial.println(xpad);

	// Update screen
	drawGame();
	if (checkLoose()) {
		debug("LOOSE");
		gameOver();
		newGame();
	}
	delay(GAME_DELAY);

	int t = millis();
	if (t - lastBallMove > BALL_DELAY) {
		moveBall();
		lastBallMove = millis();
	}
	if (digitalRead(5) == LOW) { digitalWrite(14, HIGH); }
	else {
		digitalWrite(14, LOW);
	}
	if (digitalRead(4) == LOW) { digitalWrite(15, HIGH); }
	else {
		digitalWrite(15, LOW);
	}
}

void setSprite(byte *sprite) {
	ledmatrix.clear();
	ledmatrix.drawBitmap(0,0, sprite, 8,8,255);
}

void newGame() {
	ledmatrix.clear();
	// initial position
	xball = random(0, 15);
	yball = 1;
	xpad = 7;
	direction = random(3, 6); // Go south
	for (int c = 0; c < 16; c++) {
		for (int r = 0; r < 8; r++) {
			ledmatrix.drawPixel(c, r, 255);
			delay(5);
		}
	}
}

void setPad() {
		int t = millis();
	if (digitalRead(ButtonLeft) == LOW && t - lastPressButtonLeft > 100) {
		if (xpad < 13) { xpad++; }
		lastPressButtonLeft = millis();
		Serial.println(xpad);
		if (lastState == true) {
			lastState = false;
			firstPressed = millis();
		}
		
		if (t - firstPressed > 5000 && lastState == false) {
			setup();
		}
		
	}
	if (digitalRead(ButtonRight) == LOW && t - lastPressButtonRight > 100) {
		if (xpad > 1) { xpad--; }
		lastPressButtonRight = millis();
		Serial.println(xpad);
		

	}
	if (digitalRead(ButtonLeft) == HIGH) {
		lastState = true;
	}
	
}

void debug(const char* desc) {
#ifdef DEBUG
	Serial.print(desc);
	Serial.print(" XY: ");
	Serial.print(xball);
	Serial.print(", ");
	Serial.print(yball);
	Serial.print(" XPAD: ");
	Serial.print(xpad);
	Serial.print(" DIR: ");
	Serial.println(direction);
#endif
}

int checkBounce() {
	if (!xball || !yball || xball == 15 || yball == 6) {
		int bounce = (yball == 0 || yball == 6) ? BOUNCE_HORIZONTAL : BOUNCE_VERTICAL;
#ifdef DEBUG
		debug(bounce == BOUNCE_HORIZONTAL ? "HORIZONTAL" : "VERTICAL");
#endif
		return bounce;
	}
	return 0;
}

int getHit() {
	if (yball != 6 || xball < xpad - 1 || xball > xpad + 1) {
		return HIT_NONE;
	}
	if (xball == xpad) {
		return HIT_CENTER;
	}
	return xball < xpad ? HIT_LEFT : HIT_RIGHT;
}

bool checkLoose() {
	return yball == 6 && getHit() == HIT_NONE;
}

void moveBall() {
	debug("MOVE");
	int bounce = checkBounce();
	if (bounce) {
		switch (direction) {
		case 0:
			direction = 4;
			break;
		case 1:
			direction = (bounce == BOUNCE_VERTICAL) ? 7 : 3;
			break;
		case 2:
			direction = 6;
			break;
		case 6:
			direction = 2;
			break;
		case 7:
			direction = (bounce == BOUNCE_VERTICAL) ? 1 : 5;
			break;
		case 5:
			direction = (bounce == BOUNCE_VERTICAL) ? 3 : 7;
			break;
		case 3:
			direction = (bounce == BOUNCE_VERTICAL) ? 5 : 1;
			break;
		case 4:
			direction = 0;
			break;
		}
		debug("->");
	}

	// Check hit: modify direction is left or right
	switch (getHit()) {
	case HIT_LEFT:
		if (direction == 0) {
			direction = 7;
		}
		else if (direction == 1) {
			direction = 0;
		}
		break;
	case HIT_RIGHT:
		if (direction == 0) {
			direction = 1;
		}
		else if (direction == 7) {
			direction = 0;
		}
		break;
	}

	// Check orthogonal directions and borders ...
	if ((direction == 0 && xball == 0) || (direction == 4 && xball == 15)) {
		direction++;
	}
	if (direction == 0 && xball == 15) {
		direction = 7;
	}
	if (direction == 4 && xball == 0) {
		direction = 3;
	}
	if (direction == 2 && yball == 0) {
		direction = 3;
	}
	if (direction == 2 && yball == 6) {
		direction = 1;
	}
	if (direction == 6 && yball == 0) {
		direction = 5;
	}
	if (direction == 6 && yball == 6) {
		direction = 7;
	}

	// "Corner" case
	if (xball == 0 && yball == 0) {
		direction = 3;
	}
	if (xball == 0 && yball == 6) {
		direction = 1;
	}
	if (xball == 15 && yball == 6) {
		direction = 7;
	}
	if (xball == 15 && yball == 0) {
		direction = 5;
	}

	yballOld2 = yballOld1;
	yballOld1 = yball;
	xballOld2 = xballOld1;
	xballOld1 = xball;

	if (2 < direction && direction < 6) {
		yball++;
	}
	else if (direction != 6 && direction != 2) {
		yball--;
	}
	if (0 < direction && direction < 4) {
		xball++;
	}
	else if (direction != 0 && direction != 4) {
		xball--;
	}
	xball = max(0, min(15, xball));
	yball = max(0, min(6, yball));
	debug("AFTER MOVE");
}

void gameOver() {
	ledmatrix.clear();
	ledmatrix.setCursor(0, 0);
	ledmatrix.setTextSize(1);
	ledmatrix.setTextWrap(false);  // we dont want text to wrap so it scrolls nicely
	ledmatrix.setTextColor(255);
	for (int8_t x = 0; x >= -60; x--) {
		ledmatrix.clear();
		ledmatrix.setCursor(x, 0);
		ledmatrix.print("GAME OVER");
		delay(15);
		if (digitalRead(ButtonLeft) == LOW || digitalRead(ButtonRight) == LOW) {
			goto skipGameOverAnimation;
		}

	}
	for (int8_t x = -60; x <= 45; x++) {
		ledmatrix.clear();
		ledmatrix.setCursor(x, 0);
		ledmatrix.print("GAME OVER");
		delay(10);
		if (digitalRead(ButtonLeft) == LOW || digitalRead(ButtonRight) == LOW) {
			goto skipGameOverAnimation;
		}
	}
	skipGameOverAnimation:;
}

void drawGame() {
	ledmatrix.clear();
	
	ledmatrix.drawPixel(xball,yball, 255);
	ledmatrix.drawPixel(xballOld1, yballOld1, 100);
	ledmatrix.drawPixel(xballOld2, yballOld2, 50);
	 
#ifdef DEBUG
	//Serial.println(padmap, BIN);
#endif
	
		ledmatrix.drawPixel(xpad- 1, 6, 255);
		ledmatrix.drawPixel(xpad+ 1, 6, 255);
		ledmatrix.drawPixel(xpad, 6, 255);
}




